#ifndef BASE_CPP
#define BASE_CPP

#include "basedef.h"
#include "error.h"
#include <_null.h>

const int rec_t_cnt=16;

const char* format[rec_t_cnt]={"nwwwdddb","nw\1bww","nw\2dwbd","nw\2wwwwwwwwwwwwwwwwwwbw","nw\2bbdb","nw\2wwwwwwwwwwwwwddddddb","nnw\2bbwwgwwbbg","nw\2wwwwwww","nw\2ddddddbbbbbbbbbbbbbb","nwwww","nw\2w","nn\gwwb","","","wwwwwwwwwwwwwwnn",""};
const int rec_count[rec_t_cnt]={0x53,0xD4,0x31,0x28,0x07,0x06,0x2E,0x06,0x07,0x04,0x06,0x0F,0x0F,0x2E,0x09,0x09};
const long rec_ofs_win[rec_t_cnt]={0x151C50,0x1523C8,0x152E90,0x153238,0x153990,0x153A00,0x153B68,0x154158,0x1541E0,0x154328,0x154358,0x154070,0x0,0x0,0x154398,0x0};
const long rec_ofs_dos[rec_t_cnt]={0x1F3F58,0x1F46CD,0x1F5191,0x1F5534,0x1F5C8C,0x1F5CF5,0x1F5E57,0x1F6440,0x1F64C4,0x1F6606,0x1F6636,0x1F635F,0x0,0x0,0x1F6672,0x0};
long* rec_ofs;
char* std_label[rec_t_cnt]={"TF","Tech",NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL};
char* directive[rec_t_cnt+1]={"TECHFIELDS","TECHNOLOGIES","BUILDINGS","SPECIALS","ARMOUR","SHIELDS","WEAPONS","COMPUTERS","DRIVES","UNITS","FUELCELLS","MODS","MODSABBREV","WPLURAL","SHIPS","SHIPSPLURAL","END"};

enum f_type {fdec,fbin,flbl,fnone};

f_type type_of(char fchar) {
	switch (fchar) {
		case 'b': return fdec;
		case 'd': return fdec;
		case 'w': return fdec;
		case 'f': return fbin;
		case 'g': return fbin;
		case 'n': return fnone;
	}
	return flbl;
}

int size_of(char fchar) {
	switch (fchar) {
		case 'b': return 1; // 8-bit decimal.
		case 'f': return 1; // 8-bit binary.
		case 'w': return 2;  // 16-bit decimal.
		case 'g': return 2; // 16-bit binary.
		case 'd': return 4; // 32-bit decimal.
		case 'n': return 4; // four empty byte.
	}
	return 2; // labels
}

int dec_width(int places) {
	switch (places) {
		case 1: return 3;
		case 2: return 5;
		case 4: return 10;
	}
	return 0;
}

int length(const char* f) {
	int result=0;
	int i=0;
	while (f[i]!='\0')
		result+=size_of(f[i++]);
	return result;
}

int char_to_digit(char c) {
	if (c<='9') return c-'0';
	else return c-'A'+10;
}

char digit_to_char(int i) {
	if (i<10)
		return '0'+i;
	else
		return 'A'+i-10;
}

char is_digit(char c,int base) {
	if (base<=10)
		return c>='0' && c<'0'+base;
	else
		return (c>='0' && c<='9') || (c>='A' && c<'A'-10+base);
}

int fopen(char* name,int& handle,int mode) {
	int tmp;
	asm {
		mov ah,0x3D
		mov al,byte ptr [mode]
		mov dx,[name]
		int 0x21
		jc fopen_err
		mov [tmp],ax
	}
	handle=tmp;
  return 0;
fopen_err:
	return 1;
}

void fclose(int handle) {
	asm {
		mov ah,0x3E
		mov bx,[handle]
		int 0x21
	}
}

int fseek(int handle,long pos) {
	word maj_ofs=*((word*)&pos+1);
	asm {
		mov bx,[handle]
		mov ax,0x4200
		mov dx,word ptr [pos] // the offset
		mov cx,[maj_ofs]
		int 0x21
		jc fseek_err
	}
	return 0;
fseek_err:
	return 1;
}

int fread(int handle,void* dest,int len) {
	asm {
		mov ah,0x3F
		mov bx,[handle]
		mov cx,[len]
		mov dx,[dest]
		int 0x21
		cmp ax,cx
		jnz fread_err
	}
	return 0;
fread_err:
	return 1;
}

int fwrite(int handle,void* src,int len) {
	asm {
		mov ah,0x40
		mov bx,[handle]
		mov cx,[len]
		mov dx,[src]
		int 0x21
		jc fwrite_err
	}
	return 0;
fwrite_err:
	return 1;
}

#endif