#ifndef RECORD_CPP
#define RECORD_CPP

#include <stdlib.h>
#include <string.h>
#include "basedef.h"
#include "error.h"
#include "base.cpp"
#include "globdata.cpp"
#include "input.cpp"
#include "output.cpp"

//input* in;
//output* out;

class Record {
	rec_t type;
	int num;
	void store_label(output& out,int maxlen);
	void store_name(output& out,int maxlen);
	void store_field(output& out,char fchar,data_port& port);
	err_c load_label(input& in);
	err_c load_name(input& in);
	err_c load_field(input& in,char fchar,data_port& port);
	public:
		Record(rec_t t,int n) : type(t),num(n) {};
		void store(output& out,int maxname,int maxlbl);
		err_c load(input& in);
};

void Record::store_label(output& out,int maxlen) {
	if (maxlen!=0) {
		out.put_str(labels[type]->get_label(num),maxlen);
		out << " : ";
	}
}

void Record::store_name(output& out,int maxlen) {
	out << '"';
	const char* name=names[type]->get_name(num);
	int i=0;
	int j=0;
	while (name[i]!='\0') {
		if (name[i]=='"') {
			out << '"';
			j++;
		}
		out << name[i];
		i++;
		j++;
	}
	out << '"';
	out.put_char(' ',maxlen-j);
}

void Record::store_field(output& out,char fchar,data_port& port) {
	out.set_align(align_right);
	switch (type_of(fchar)) {
		case fdec:
			out.put(',');
			out.put_num(port.request(size_of(fchar)),dec_width(size_of(fchar)));
			break;
		case fbin:
			out.put(',');
			out.put_flags(port.request(size_of(fchar)),size_of(fchar)*8);
			break;
		case flbl: {
			out.put(',');
			word val=port.request(size_of(fchar));
			char* lbl=labels[fchar-1]->get_label(val);
			if (lbl!=NULL) out.put_str(lbl,max_lbl_len);
			else out.put_num(val,max_lbl_len);
			break;
		}
		case fnone: {
			port.seek(size_of(fchar));
			break;
		}
	}
}

void Record::store(output& out,int maxname,int maxlbl) {
	out.set_align(align_left);
	store_label(out,maxlbl);
	store_name(out,maxname);
	out.set_align(align_right);
	data_port port=rdata[type]->request_port(num);
	int f_pos=0;
	while (format[type][f_pos]!='\0') {
		store_field(out,format[type][f_pos],port);
		f_pos++;
	}
	out << ";\n";
}


// Load routines.

err_c Record::load_label(input& in) {
	in.skip_ws();
	if (in.peek()=='"') {
		labels[type]->add_label(NULL);
		return NO_ERROR;
	}
	char* s=new char[max_lbl_len+1];
	if (!in.get_token(s,max_lbl_len)) {
		labels[type]->add_label(s);
		if (in.get_char(':')) return COLON_EXPECTED;
		return NO_ERROR;
	}
	delete s;
	return LABEL_OR_NAME_EXPECTED;
}

err_c Record::load_name(input& in) {
	if (in.get_char('"')) return QUOTE_EXPECTED;
	char* name=(char*)malloc(max_name_len);
	if (!in.get_string(name,max_name_len,'"',1)) {
		name=(char*)realloc(name,strlen(name)+1);
		names[type]->add_name(name);
		return NO_ERROR;
	}
	free(name);
	return QUOTE_EXPECTED;
}

err_c Record::load_field(input& in,char fchar,data_port& port) {
	dword val;
	switch (type_of(fchar)) {
		case fdec: {
			if (in.get_char(',')) return COMMA_EXPECTED;
			if (in.get_num(val,10)) return DECIMAL_EXPECTED;
			break;
		}
		case fbin: {
			if (in.get_char(',')) return COMMA_EXPECTED;
			if (in.get_num(val,2)) return BINARY_EXPECTED;
			break;
		}
		case flbl: {
			if (in.get_char(',')) return COMMA_EXPECTED;
			char s[max_lbl_len+1];
			if (in.get_token(s,max_lbl_len)==0) {
				val=labels[fchar-1]->find(s,rec_count[fchar-1]);
				if (val==rec_count[fchar-1]) return UNKNOWN_LABEL;
			}
			else
				if (in.get_num(val,10)) return LABEL_OR_DECIMAL_EXPECTED;
			break;
		}
		case fnone: {
			if (version==v0_2) {
				if (in.get_char(',')) return COMMA_EXPECTED;
				if (in.get_num(val,10)) return DECIMAL_EXPECTED;
			}
			port.seek(size_of(fchar));
			return NO_ERROR;
		}
	}
	port.upload(val,size_of(fchar));
	return NO_ERROR;
}

err_c Record::load(input& in) {
	safe_call(load_label(in));
	safe_call(load_name(in));
	int f_pos=0;
	data_port port=rdata[type]->request_port(num);
	while (format[type][f_pos]!='\0') {
		safe_call(load_field(in,format[type][f_pos],port));
		f_pos++;
	}
	if (in.get_char(';')) return SEMICOLON_EXPECTED;
	return NO_ERROR;
}

#endif